import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.gridspec import GridSpec

from config import Config, FiguresConfig, DataConfig


def combined_vis():
    perf_data = pd.read_csv(f"{Config.JaspOutputPath}/single_tasks.csv", sep=";")
    subj_data = pd.read_csv(f"{Config.JaspOutputPath}/aggregated_tasks.csv", sep=";")

    fig_size = (
        FiguresConfig.FullColumn,
        FiguresConfig.PageHeight * 0.1675
    )
    margin = FiguresConfig.calc_margin_single(fig_size=fig_size, absolute_margin=(1.1, .9, 0.075, 0.875))
    spacing = FiguresConfig.calc_spacing_single(fig_size=fig_size, absolute_spacing=(200, 2))

    fig = plt.figure(
        figsize=fig_size,
        dpi=FiguresConfig.Dpi
    )
    gs = GridSpec(
        nrows=2,
        ncols=25,
        figure=fig,
        top=margin["top"],
        right=margin["right"],
        left=margin["left"],
        bottom=margin["bottom"],
        wspace=spacing["vertical"],
        hspace=spacing["horizontal"],
    )

    def _draw_plot(ax: plt.Axes, data: pd.DataFrame, visKey: str, dvKey: str, drawXLabel: bool):
            for aKey in ["Non-Adapted", "Adapted"]:
                sns.regplot(
                    ax=ax,
                    data=data[(data["ChartType"] == visKey) & (data["Adaptation"] == aKey)],
                    x=f"VL-{visKey}",
                    y=f"{dvKey} {visKey}",
                    ci=95,
                    color=FiguresConfig.Palettes["Adaptation"][aKey],
                    scatter=False
                )

            for spine in ax.spines.values():
                spine.set_color(FiguresConfig.Palettes["ChartType"][visKey])
                spine.set_linewidth(FiguresConfig.BorderWidth)

            if not drawXLabel:
                ax.set_xlabel("")
            else:
                ax.set_xlabel(FiguresConfig.AxisLabels["VL"])

            ylim = DataConfig.calc_fixed_value_limits(dvKey)
            ax.set_ylim(
                bottom=ylim[0],
                top=ylim[1]
            )
            ax.set_yticks(DataConfig.FixedValues[dvKey])
            ax.set_ylabel(FiguresConfig.AxisLabels[dvKey])

            xlim = DataConfig.calc_fixed_value_limits("VL")
            ax.set_xlim(
                left=xlim[0],
                right=xlim[1]
            )
            ax.set_xticks(DataConfig.FixedValues["VL"])

    for (y, visKey) in enumerate(["Bar", "Scatter"]):
        for (x, dvKey) in enumerate(["Time", "Accuracy"]):
            ax = fig.add_subplot(gs[y, x * 5: (x + 1) * 5])

            _draw_plot(
                ax=ax,
                data=perf_data,
                visKey=visKey,
                dvKey=dvKey,
                drawXLabel=y == 1,
            )

        for (x, dvKey) in enumerate(["Dependability", "Usefulness", "Intuitive Use"]):
            x = x + 2
            ax = fig.add_subplot(gs[y, x * 5: (x + 1) * 5])

            _draw_plot(
                ax=ax,
                data=subj_data,
                visKey=visKey,
                dvKey=dvKey,
                drawXLabel=y == 1,
            )

    FiguresConfig.set_custom_legend(
        fig=fig,
        keys=["Bar Chart", "Scatter Plot"],
        colors=[
            FiguresConfig.Palettes["ChartType"]["Bar"],
            FiguresConfig.Palettes["ChartType"]["Scatter"],
        ],
        title="Visualization Type",
        loc="upper right",
        bbox_to_anchor_y=1.0245
    )
    FiguresConfig.set_custom_legend(
        fig=fig,
        keys=["Non-Adapted", "Adapted"],
        colors=[
            FiguresConfig.Palettes["Adaptation"]["Non-Adapted"],
            FiguresConfig.Palettes["Adaptation"]["Adapted"],
        ],
        title="Adaptation",
        loc="upper left",
        bbox_to_anchor_y=1.0245
    )

    fig.align_ylabels()
    fig.tight_layout()
    for format_str in FiguresConfig.VisFormats:
        fig.savefig(f"{Config.PaperContentOutputPath}/combined vis.{format_str}")
    plt.close(fig)


def vl_scores():
    data = pd.read_csv(f"{Config.JaspOutputPath}/aggregated_tasks.csv", sep=";")
    data = data[["PID", "VL-Bar", "VL-Scatter"]]
    data = data.groupby("PID", axis=0).mean().reset_index()

    fig_size = (
        FiguresConfig.HalfColumn,
        FiguresConfig.PageHeight * 0.1675
    )
    margin = FiguresConfig.calc_margin_single(fig_size=fig_size, absolute_margin=(1.15, .9, 1, 0.875))
    spacing = FiguresConfig.calc_spacing_single(fig_size=fig_size, absolute_spacing=(0.4, 2))

    fig = plt.figure(
        figsize=fig_size,
        dpi=FiguresConfig.Dpi
    )
    gs = GridSpec(
        nrows=1,
        ncols=2,
        figure=fig,
        left=margin["left"],
        top=margin["top"],
        right=margin["right"],
        bottom=margin["bottom"],
        wspace=fig_size[0] * spacing["vertical"],
        hspace=fig_size[1] * spacing["horizontal"]
    )

    for (x, visKey) in enumerate(["Bar", "Scatter"]):
        ax = fig.add_subplot(gs[0, x])
        twin_ax = ax.twinx()

        sns.kdeplot(
            ax=ax,
            data=data,
            x=f"VL-{visKey}",
            color=FiguresConfig.Palettes["ChartType"][visKey],
            linewidth=5
        )

        ylim = DataConfig.calc_fixed_value_limits("KDE-VL")
        ax.set_ylim(
            bottom=ylim[0],
            top=ylim[1]
        )
        ax.set_yticks(DataConfig.FixedValues["KDE-VL"])

        xlim = DataConfig.calc_fixed_value_limits("VL-Scores")
        ax.set_xlim(
            left=xlim[0],
            right=xlim[1]
        )
        ax.set_xticks(DataConfig.FixedValues["VL-Scores"])

        sns.histplot(
            ax=twin_ax,
            data=data,
            x=f"VL-{visKey}",
            color=FiguresConfig.Palettes["ChartType"][visKey],
            bins=14,
            alpha=.6,
            binwidth=.035,
            linewidth=0
        )

        ylim = DataConfig.calc_fixed_value_limits("Hist-VL")
        twin_ax.set_ylim(
            bottom=ylim[0],
            top=ylim[1]
        )
        twin_ax.set_yticks(DataConfig.FixedValues["Hist-VL"])

        xlim = DataConfig.calc_fixed_value_limits("VL-Scores")
        twin_ax.set_xlim(
            left=xlim[0],
            right=xlim[1]
        )
        twin_ax.set_xticks(DataConfig.FixedValues["VL-Scores"])

        twin_ax.grid(False)

    FiguresConfig.set_custom_legend(
        fig=fig,
        keys=["Bar Chart", "Scatter Plot"],
        colors=[
            FiguresConfig.Palettes["ChartType"]["Bar"],
            FiguresConfig.Palettes["ChartType"]["Scatter"],
        ],
        title="Visualization Type",
        loc="upper center",
        bbox_to_anchor_y=1.022
    )

    fig.align_ylabels()
    fig.tight_layout()
    for format_str in FiguresConfig.VisFormats:
        fig.savefig(f"{Config.PaperContentOutputPath}/vl score vis.{format_str}")
    plt.close(fig)


if __name__ == '__main__':
    FiguresConfig.set_rc_plot_values()

    with sns.axes_style(FiguresConfig.SnsStyle):
        combined_vis()
        vl_scores()
